#include <utility.h>
/***************************************************************************/
/*  Copyright 1995 National Instruments Corporation.  All Rights Reserved. */
/***************************************************************************/

#include "nivxi.h"
#include "gpib.h"
#include "formatio.h"
#include "eip114x.h"

#define EIP114X_REVISION "EIP114x V1.0 12/94"
#define NO_ERROR     0
#define TRUE         1
#define FALSE        0
#define BIT_ERRQ  0x04
#define BIT_MAV   0x10

/*= EIP Microwave Inc. 1140A/1141A/1142A VXIbus Microwave Synthesizers ====*/
/* LabWindows 2.3a Instrument Driver                                       */
/* Original Release: January 1995                                          */
/* By: Jim McDaniel, EIP Microwave, Milpitas, CA 95035                     */
/*     Ph (408) 945-1477    Fax (408) 945-0977                             */
/* Originally written in C                                                 */
/* Structural Notes:                                                       */
/*     Since GENERAL PURPOSE FUNCTIONS verify the instrument ID and the    */
/*     fact that the instrument has been opened, they may be exported.     */
/*     Since UTILITY FUNCTIONS do not, they may not be exported.           */
/*     The function eip114x_write_bytes tests the validity of              */
/*     Instrument_ID and calls eip114x_closed to see that the instrument   */
/*     driver has been opened. This was done to save code size.            */
/*                                                                         */
/* For LabWindows Technical Support Call, National Instruments, Austin, TX */
/*     Ph (800) 433-3488    Fax (512) 794-5794                             */
/*                                                                         */
/* Modification History:                                                   */
/* Date     Author, Company,                                               */
/*          Description                                                    */
/*                                                                         */
/* 2/17/95  Jim McDaniel, MCD Engineering                                  */
/*          Removed secondary functions.                                   */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/*=========================================================================*/

/*= INSTRUMENT TABLES =====================================================*/
/* Since there may be more than one eip114x in the system, the driver uses */
/* a set of tables to remember the parameters needed to identify each one. */
/* The global variable Instrument_ID, provided by eip114x_init(), is the   */
/* index used to access the tables.                                        */
/*                                                                         */
/* base_addr[]  contains the VXI base address in A16 space for the instr.  */
/* bd[]         contains the device descriptor returned by OpenDev         */
/*              if the instrument is accessed via GPIB. (NI-GPIB only)     */
/* interface[]  specifies the type of interface (NI-VXI or NI-GPIB).       */
/* instr_cnt    contains the number of instruments open of this model type.*/
/* log_addr[]   contains the logical addresses of opened instruments.      */
/* model[]      differentiates the prticular model (1140,1141 etc.)        */
/*=========================================================================*/
static long base_addr[eip114x_MAX_INSTR + 1];
static int bd[eip114x_MAX_INSTR + 1];
static int interface[eip114x_MAX_INSTR + 1];
static int instr_cnt;
static int log_addr[eip114x_MAX_INSTR + 1];
static int model[eip114x_MAX_INSTR + 1];

/*= STATIC DATA ===========================================================*/
/* cmd[]        a buffer for GPIB and VXI I/O strings.                     */
/* eip114x_err: the global error variable for the instrument module        */
/* timeout[]:   a look up table used to associate GPIB timeout codes with  */
/*              Word Serial timout codes.                                  */
/* bytes:       contains the actual number of bytes transferred by WSrd,   */
/*              WSwrt, WSrdf, & WSwrtf.                                    */
/* ibcnt:       contains the number of bytes transferred by GPIB reads and */
/*              writes. See the GPIB library I/O Class for more information*/
/*              MaxFreq[3] Maximum frequency for each model                */
/*              MinFreq[3] Minimum frequency for each model                */
/*=========================================================================*/

#define CMDLENGTH                  149
#define EIP114X_ERR_STATUS         300
#define EIP114X_ERR_PARAM_COMPAT   301

#define EIP_MODEL_1140               0
#define EIP_MODEL_1141               1
#define EIP_MODEL_1142               2

static  long   bytes;
static  char   cmd[CMDLENGTH+1];
static  char   *CmdString[115];
static  int    eip114x_err;
static  int    eip114x_auto_poll;

static  char   *kw_binary[2];
static  char   *kw_ext_int[2];
static  char   *kw_dir[2];
static  char   *kw_link_trig[5];
static  char   *kw_mode[5];
static  char   *kw_nv_alt[5];
static  char   *kw_triggers[11];
static  char   *kw_unit_freq[4];

static  double MaxFreq[3];
static  double MinFreq[3];
static  double MaxPower[3];
static  double MinPower[3];
static  double MaxAtten[3];
static  double MinAtten[3];
static  char   *msg_cal[5];
static  char   *msg_self_test[6];
static  long   timeout[18];
static  double unit_freq_mult[4];

/*= UTILITY ROUTINES ======================================================*/
int   eip114x_close_instr (int);
int   eip114x_device_closed (int);
int   eip114x_get_status_reg (int, int *);
int   eip114x_init_gpib (int);
int   eip114x_invalid_integer_range (int, int, int, int);
int   eip114x_invalid_longint_range (long, long, long, int);
int   eip114x_invalid_real_range (double, double, double, int);
int   eip114x_open_instr (int, int, int *);
int   eip114x_poll (int, int *);
int   eip114x_read_data (int, char *, int);
int   eip114x_read_data_file (int, char *);
int   eip114x_read_reg (int, int, int *);
int   eip114x_read_int (int, int *);
int   eip114x_set_timeout (int, int);
void  eip114x_setup_arrays (void);
int   eip114x_test_instr_status (int Instrument_ID);
int   eip114x_write_bytes (int, char *, int);
int   eip114x_write_data (int, char *, int);
int   eip114x_write_data_file (int, char *);
int   eip114x_write_reg (int, int, int);

/*=========================================================================*/
/* Function: Initialize                                                    */
/* Purpose:  This function opens the instrument, queries the instrument    */
/*           for its ID, and initializes the instrument to a known state.  */
/*=========================================================================*/
int eip114x_init (int laddr, int platform, int id_query, int rest,
        int *Instrument_ID, int *Model_Number)
{
   int ID, value;
   if (eip114x_invalid_integer_range (laddr, 0, 255, -1) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (platform, 1, 2, -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (id_query, 0, 1, -3) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (rest, 0, 1, -4) != 0)
      return eip114x_err;

     /* INITIALIZE ENTRY IN INSTRUMENT TABLE AND INTERFACE FOR INSTRUMENT.*/
   if (eip114x_open_instr (laddr, platform, &ID) != 0)
      return eip114x_err;

     /*  MESSAGE BASED ID Query  */
   if (id_query) {  /* USER WANTS TO VERIFY THE INSTRUMENT MODEL, SO   */
                    /* READ THE INSTRUMENT MODEL FROM THE VXI REGISTER */
      if (eip114x_read_reg (ID, 2, Model_Number) != 0) {
         eip114x_close_instr (ID);
         eip114x_err = 231;
         return eip114x_err;
      }
      value &= 0xfff;
      switch(*Model_Number) {
         case 1140:
               model[ID] = 0;
               break;
         case 1141:
               model[ID] = 1;
               break;
         case 1142:
               model[ID] = 2;
               break;
         default:
               eip114x_close_instr (ID);
               eip114x_err = 223;
               return eip114x_err;
               break;
      }
   }
   else /* USER DID NOT SELECT ID QUERY */
      model[ID] = 0;

   if (rest) {
        /*  INITIALIZE THE INSTRUMENT TO A KNOWN STATE.  */
      if (eip114x_write_data (ID, "*RST", 4) != 0) {
         eip114x_close_instr (ID);
         return eip114x_err;
      }
   }
     /* INITIALIZE INSTRUMENT-DEPENDENT COMMAND ARRAYS.  */
   eip114x_setup_arrays ();
     /*  SET TIMEOUT TO 10 SECONDS  */
   eip114x_set_timeout (ID, 13);

   *Instrument_ID = ID;
   return eip114x_err;
}

/*=========================================================================*/
/* Name:         eip114x_am_depth_step                                     */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_am_depth_step (int Instrument_ID, int AM_Depth_Step)
{
   if (eip114x_invalid_integer_range (AM_Depth_Step, -100, 100, -2) != 0)
      return eip114x_err;

   Fmt (cmd, "%s<AM:STEP %d", AM_Depth_Step);
   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
      return eip114x_err;

   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}

/*=========================================================================*/
/* Name:        eip114x_am_setup                                           */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_am_setup (int Instrument_ID, int AM_Depth, int AM_State)
{
   int cnt;
   if (eip114x_invalid_integer_range (AM_Depth, 0, 100, -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (AM_State,-1,   1, -3) != 0)
      return eip114x_err;

   Fmt (cmd, "%s<AM:DEPT %d", AM_Depth);
   cnt = NumFmtdBytes ();
   if (AM_State != -1) {
      Fmt (cmd, "%s[a]<;STAT %d", AM_State);
      cnt = cnt + NumFmtdBytes ();
   }
   if (eip114x_write_bytes (Instrument_ID, cmd, cnt) != 0)
       return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_cal_all                                            */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_cal_all (int Instrument_ID)
{
   if (eip114x_write (Instrument_ID, "CAL?") != 0)
      return eip114x_err;

   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}

/*=========================================================================*/
/* Name:        eip114x_cal_all_query                                      */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_cal_all_query (int Instrument_ID, int *Result_Code,
        char *Result_Message)
{
   int msg_num,n,tr,bit[5];
   if (eip114x_read_int (Instrument_ID, Result_Code) != 0)
      return eip114x_err;
   if (*Result_Code !=0) {
      tr = *Result_Code;
      n=16;
      msg_num=4;
      while(n > 0) {
         if ((tr-n) >= 0) {
               bit[msg_num]=TRUE;
               tr=tr-n;
         }
         else {
               bit[msg_num]=FALSE;
         }
         n=n/2;
         msg_num=msg_num-1;
      }
      Fmt(Result_Message,"");
      for (msg_num = 0; msg_num<5; msg_num=msg_num+1) {
         if (bit[msg_num] == TRUE)
               Fmt(Result_Message, "%s[a]<%s;", msg_cal[msg_num]);
         }
   }
   else {
      Fmt(Result_Message, "%s<%s", "Calibration Successful");
   }
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_cls                                                */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_cls (int Instrument_ID)
{
   if (eip114x_write_bytes (Instrument_ID, "*CLS", 4) != 0)
      return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_cm_setup                                           */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_cm_setup (int Instrument_ID, int CM_State, double IF_Frequency)
{
   int cnt;
   if (eip114x_invalid_integer_range (CM_State, -1, 1, -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_real_range (IF_Frequency, 300.0 , 1000.0, -3) != 0)
      return eip114x_err;

   Fmt (cmd, "%s<CM:FREQ %f[p1]", IF_Frequency*1.0E6);
   cnt = NumFmtdBytes ();
   if (CM_State != -1) {
      Fmt (cmd, "%s[a]<;STAT %d", CM_State);
      cnt += NumFmtdBytes ();
   }
  if (eip114x_write_bytes (Instrument_ID, cmd, cnt) != 0)
       return eip114x_err;
  if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

  return eip114x_err;
}

/*=========================================================================*/
/* Name:        eip114x_error_polling                                      */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_error_polling (int Auto_Polling)
{
   if (eip114x_invalid_integer_range (Auto_Polling, 0, 1, -1) != 0)
      return eip114x_err;

   eip114x_auto_poll = Auto_Polling;

   return eip114x_err;
}

/*=========================================================================*/
/* Name:        eip114x_frequency                                          */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_frequency (int Instrument_ID, int Tuning_Method, int Parameter_Type,
        double Frequency, int Unit)
{
   int cnt, keyword;
   cnt = 0;

   if (eip114x_invalid_integer_range (Tuning_Method, 0, 1, -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Parameter_Type, 0, 4, -3) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Unit, 0, 3, -5) != 0)
      return eip114x_err;
   if (Tuning_Method == 0) {
      if (eip114x_invalid_real_range (Frequency * unit_freq_mult[Unit],
                                    MinFreq[model[Instrument_ID]],
                                    MaxFreq[model[Instrument_ID]], -4) != 0)
      return eip114x_err;
   }
   else {
      if (eip114x_invalid_real_range (Frequency * unit_freq_mult[Unit], -1.0e6,+1.0E6, -4) != 0)
      return eip114x_err;
   }

   if (Parameter_Type == 0) {
      if (Tuning_Method == 1) {
         Fmt (cmd, "%s<FREQ:FTUN %f[p1]", (Frequency * unit_freq_mult[Unit]));
         cnt += NumFmtdBytes ();
      }
      else {
         Fmt (cmd, "%s<FREQ %f[p1]",(Frequency * unit_freq_mult[Unit]));
         cnt += NumFmtdBytes ();
      }
   }
   else {
      if (Tuning_Method == 1) {
         eip114x_err = EIP114X_ERR_PARAM_COMPAT;
         return eip114x_err;
      }
      else {
         Fmt (cmd, "%s<FREQ %s", kw_nv_alt[Parameter_Type]);
         cnt = NumFmtdBytes ();
      }
   }
   if (eip114x_write_bytes (Instrument_ID, cmd, cnt) != 0)
       return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}

/*=========================================================================*/
/* Name:        eip114x_freq_power                                         */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_freq_power (int Instrument_ID, double Frequency, int Unit,
        double Power, int Attenuation)
{
   int cnt;/* Number of bytes to be sent to the instrument */
   if (eip114x_invalid_integer_range (Unit, 0, 3, -3) != 0)
      return eip114x_err;
   if (eip114x_invalid_real_range (Frequency * unit_freq_mult[Unit],
                                 MinFreq[model[Instrument_ID]],
                                 MaxFreq[model[Instrument_ID]], -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_real_range (Power,
                                 MinPower[model[Instrument_ID]],
                                 MaxPower[model[Instrument_ID]], -4) != 0)
   if (eip114x_invalid_real_range (Attenuation, -1,
                                 MaxAtten[model[Instrument_ID]], -5) != 0)
      return eip114x_err;

   Fmt (cmd, "%s<FREQ %f[p1]", Frequency * unit_freq_mult[Unit]);
   cnt = NumFmtdBytes ();
   if (Attenuation < 0) {
      Fmt (cmd, "%s[a]<;:POW:ATT:AUTO 1");
      cnt += NumFmtdBytes ();
   }
   else {
      Fmt (cmd, "%s[a]<;:POW:ATT %d", Attenuation);
      cnt += NumFmtdBytes ();
   }
   Fmt (cmd, "%s[a]<;:POW %f", Power);
   cnt += NumFmtdBytes ();
   if (eip114x_write_bytes (Instrument_ID, cmd, cnt) != 0)
       return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_frequency_step                                     */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_frequency_step (int Instrument_ID, double Frequency_Step, int Unit)
{
   double FStep;
   if (eip114x_invalid_integer_range (Unit, 0, 3, -3) != 0)
      return eip114x_err;

   FStep = Frequency_Step * unit_freq_mult[Unit];

   if (eip114x_invalid_real_range (FStep, MinFreq[model[Instrument_ID]],
                                  MaxFreq[model[Instrument_ID]], -2) != 0)

   Fmt (cmd, "%s<FREQ:STEP %f[p1]", FStep);
   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_list_pvector_values                                */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_list_pvector_values (int Instrument_ID, int Point_Number,
        double Frequency, int Unit, double Power, int AM_Depth, int CM_State,
        int PULM_State)
{

   if (eip114x_invalid_integer_range (Point_Number, 1, 201, -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Unit, 0, 3, -4) != 0)
      return eip114x_err;
   if (eip114x_invalid_real_range (Frequency * unit_freq_mult[Unit],
                                   MinFreq[model[Instrument_ID]],
                                   MaxFreq[model[Instrument_ID]], -3) != 0)
      return eip114x_err;
   if (eip114x_invalid_real_range (Power, MinPower[model[Instrument_ID]],
                                   MaxPower[model[Instrument_ID]], -5) != 0)
   if (eip114x_invalid_integer_range (AM_Depth, 0, 100, -6) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (CM_State, 0, 1, -7) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (PULM_State, 0, 1, -8) != 0)
      return eip114x_err;

   Fmt (cmd,"%s<:LIST:PVEC %d,%f%s,%f,%d,%d,%d ", Point_Number, Frequency,
        kw_unit_freq[Unit], Power, AM_Depth, CM_State, PULM_State);

   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_list_setup                                         */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_list_setup (int Instrument_ID, int Starting_Event, int Count,
        int Direction, int Dwell_Link, double Dwell_Time)

{
   if (eip114x_invalid_integer_range (Starting_Event, 1, 4, -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Count, 1, 32000, -3) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Direction, 0, 1, -4) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Dwell_Link, 0, 4, -5) != 0)
      return eip114x_err;
   if (eip114x_invalid_real_range (Dwell_Time, 5.0e-3, 32000, -6) != 0)
      return eip114x_err;

   Fmt (cmd,"%s<LIST:STAR %s;COUN %d;DIR %s;PVEC:DWEL %f;DWEL:LINK %s",
        kw_link_trig[Starting_Event], Count, kw_dir[Direction], Dwell_Time,
        kw_link_trig[Dwell_Link]);

   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_memory_free_all_qry                                */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_memory_free_all_qry (int Instrument_ID, int *Free_bytes)
{
   if (eip114x_write_bytes (Instrument_ID, "MEM:FREE?", 9) != 0)
       return eip114x_err;

   if (eip114x_read_data (Instrument_ID, cmd, 20) == 0) {
      if (Scan (cmd, "%s>%d", Free_bytes) != 1) {
         eip114x_err = 236;
         return eip114x_err;
      }
   }
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}

/*=========================================================================*/
/* Name:        eip114x_memory_list_delete                                 */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_memory_list_delete (int Instrument_ID)
{
   if (eip114x_write_bytes (Instrument_ID, "MEM:DEL LIST", 12) != 0)
      return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}

/*=========================================================================*/
/* Name:        eip114x_mode_set                                           */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_mode_set (int Instrument_ID, int List_Mode, int Frequency_Mode,
        int Power_Mode, int AM_Mode, int CM_Mode, int PULM_Mode)
{
   int cnt;
   cnt = 0;
   if (eip114x_invalid_integer_range (List_Mode, 0, 2, -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Frequency_Mode, 0, 3, -3) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Power_Mode, 0, 4, -4) != 0)
      return eip114x_err;
   if (Power_Mode == 3) {/* Power_mode can't be SWEep1 */
      eip114x_err = -4;
      return eip114x_err;
   }
   if (eip114x_invalid_integer_range (AM_Mode, 0, 2, -5) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (CM_Mode, 0, 2, -6) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (PULM_Mode, 0, 2, -7) != 0)
      return eip114x_err;

   cmd[0]='\0';   /* MAKE cmd A NULL STRING */
   if (List_Mode == 2) {
     /* LIST:PVEC:MODE LIST sets them all to LIST mode */
      Fmt (cmd, "%s<LIST:PVEC:MODE %s", kw_mode[2]);
      cnt += NumFmtdBytes ();
   }
   else {
      if (List_Mode == 1) {/* FIXed Mode */
         Fmt (cmd, "%s<LIST:PVEC:MODE %s", kw_mode[List_Mode]);
         cnt += NumFmtdBytes ();
      }
      if (Frequency_Mode > 0) {
         Fmt (cmd, "%s[a]<;:FREQ:MODE %s", kw_mode[Frequency_Mode]);
         cnt += NumFmtdBytes ();
      }
      if (Power_Mode > 0) {
         Fmt (cmd, "%s[a]<;:POW:MODE %s", kw_mode[Power_Mode]);
         cnt += NumFmtdBytes ();
      }
      if (AM_Mode > 0) {
         Fmt (cmd, "%s[a]<;:AM:MODE %s", kw_mode[AM_Mode]);
         cnt += NumFmtdBytes ();
      }
      if (CM_Mode > 0) {
         Fmt (cmd, "%s[a]<;:CM:MODE %s", kw_mode[CM_Mode]);
         cnt += NumFmtdBytes ();
      }
      if (PULM_Mode > 0) {
         Fmt (cmd, "%s[a]<;:PULM:MODE %s", kw_mode[PULM_Mode]);
         cnt += NumFmtdBytes ();
      }
   }
   if (eip114x_write_bytes (Instrument_ID, cmd, cnt) != 0)
       return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_outp_filter_center                                 */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_outp_filter_center (int Instrument_ID)
{
   if (eip114x_write_bytes (Instrument_ID, "OUTP:FILT:CENT", 14) != 0)
      return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_outp_rosc                                          */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_outp_rosc (int Instrument_ID, int Reference_Source,
        int Connector_Direction)
{
   if (eip114x_invalid_integer_range (Connector_Direction, 0, 1, -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Reference_Source,    0, 1, -3) != 0)
      return eip114x_err;

   if (Connector_Direction==1 && Reference_Source==1) {
      eip114x_err = EIP114X_ERR_PARAM_COMPAT;
      return eip114x_err;
   }
   Fmt (cmd, "%s<ROSC:SOUR %s;:OUTP:ROSC %s", kw_ext_int[Reference_Source],
        kw_binary[Connector_Direction]);

   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_outp_state                                         */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_rf_outp_control (int Instrument_ID, int alc_state, int blanking_state, int RF_Output)
{

   if (eip114x_invalid_integer_range (alc_state, 0, 1, -2) != 0)
       return eip114x_err;
   if (eip114x_invalid_integer_range (blanking_state, 0, 1, -2) != 0)
       return eip114x_err;
   if (eip114x_invalid_integer_range (RF_Output, 0, 1, -2) != 0)
      return eip114x_err;

   Fmt (cmd, "%s<POW:ALC %d;:OUTP:BLAN %s;:OUTP %s",
      alc_state, kw_binary[blanking_state], kw_binary[RF_Output]);
   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;

   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_outp_trig                                          */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_outp_trig (int Instrument_ID, int Trigger_Line, int New_State)
{
   if (eip114x_invalid_integer_range (Trigger_Line, 1, 10, -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (New_State,    0,  2, -3) != 0)
      return eip114x_err;

   if (New_State==2)
      Fmt (cmd,"%s<OUTP:%s:IMM", kw_triggers[Trigger_Line]);
   else
      Fmt (cmd,"%s<OUTP:%s %d", kw_triggers[Trigger_Line], New_State);

   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_power                                              */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_power (int Instrument_ID, double Power)
{
   if (eip114x_invalid_real_range (Power, MinPower[model[Instrument_ID]],
                                   MaxPower[model[Instrument_ID]], -2) !=0 )
      return eip114x_err;
   Fmt (cmd, "%s<:POW %f", Power);
   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;

   return eip114x_err;
}

/*=========================================================================*/
/* Name:        eip114x_power_dac                                          */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_power_dac (int Instrument_ID, int Percent_Power)
{
   if (eip114x_invalid_integer_range (Percent_Power, 0, 100, -2) != 0)
      return eip114x_err;

   Fmt (cmd, "%s<POW:DAC %d", Percent_Power);
   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_pulm_setup                                         */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_pulm_setup (int Instrument_ID, int PULM_State)
{
   if (eip114x_invalid_integer_range (PULM_State, 0, 1, -2) != 0)
      return eip114x_err;

   Fmt (cmd, "%s<PULM %d", PULM_State);
   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}

/*=========================================================================*/
/* Name:        eip114x_status_enable_srq                                  */
/* Description: Programs all the 488.2 status enable registers in the      */
/*            instrument. Takes register values as strings so they can     */
/*            be expressed as hex values on a user panel.                  */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_status_enable_srq(int Instrument_ID, int Quest_Enable,
        int Oper_Enable, int Std_Enable, int Oper_Ptr, int Oper_Ntr,
        int SRQ_Enable)
{
   if (eip114x_invalid_integer_range (Quest_Enable, 0, 0x7fff, -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Oper_Enable, 0, 0x7fff, -3) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Std_Enable, 0, 0xff, -4) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Oper_Ptr, 0, 0x7fff, -5) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Oper_Ntr, 0, 0x7fff, -6) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (SRQ_Enable, 0, 0xff, -7) != 0)
      return eip114x_err;

   Fmt (cmd,"STAT:QUES:ENAB %d;:STAT:OPER:NTR %d;PTR %d;ENAB %d;*ESE %d;*SRE %d",
         Quest_Enable, Oper_Ntr, Oper_Ptr, Oper_Enable, Std_Enable,
         SRQ_Enable);

   if (eip114x_err == 0)
      if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
          return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}

/*=========================================================================*/
/* Name:        eip114x_status_std_read_all                                */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_status_std_read_all (int Instrument_ID, int *Standard_Event,
        int *Enable, int *Status_Byte, int *SRQ_Enable)
{
   if (eip114x_write_bytes (Instrument_ID, "*ESR?", 5) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Standard_Event) != 0)
       return eip114x_err;

   if (eip114x_write_bytes (Instrument_ID, "*ESE?", 5) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Enable) != 0)
       return eip114x_err;

   if (eip114x_write_bytes (Instrument_ID, "*STB?", 5) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Status_Byte) != 0)
       return eip114x_err;

   if (eip114x_write_bytes (Instrument_ID, "*SRE?", 5) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, SRQ_Enable) != 0)
       return eip114x_err;

   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}

/*=========================================================================*/
/* Name:        eip114x_status_oper_read_all                               */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_status_oper_read_all (int Instrument_ID, int *Condition, int *Ntr,
        int *Ptr, int *Event, int *Enable)
{
   if (eip114x_write_bytes (Instrument_ID, "STAT:OPER:COND?", 15) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Condition) != 0)
       return eip114x_err;

   if (eip114x_write_bytes (Instrument_ID, "STAT:OPER:NTR?", 14) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Ntr) != 0)
       return eip114x_err;

   if (eip114x_write_bytes (Instrument_ID, "STAT:OPER:PTR?", 14) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Ptr) != 0)
       return eip114x_err;

   if (eip114x_write_bytes (Instrument_ID, "STAT:OPER?", 10) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Event) != 0)
       return eip114x_err;

   if (eip114x_write_bytes (Instrument_ID, "STAT:OPER:ENAB?", 15) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Enable) != 0)
       return eip114x_err;

   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_status_quest_read_all                              */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_status_quest_read_all (int Instrument_ID, int *Cal_Condition,
        int *Event, int *Enable, int *Frequency_Event, int *Power_Event,
        int *Voltage_Event)
{
   if (eip114x_write_bytes (Instrument_ID, "STAT:QUES:CAL:COND?", 19) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Cal_Condition) != 0)
       return eip114x_err;

   if (eip114x_write_bytes (Instrument_ID, "STAT:QUES?", 10) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Event) != 0)
       return eip114x_err;

   if (eip114x_write_bytes (Instrument_ID, "STAT:QUES:ENAB?", 15) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Enable) != 0)
       return eip114x_err;

   if (eip114x_write_bytes (Instrument_ID, "STAT:QUES:FREQ?", 15) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Frequency_Event) != 0)
       return eip114x_err;

   if (eip114x_write_bytes (Instrument_ID, "STAT:QUES:POW?", 14) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Power_Event) != 0)
       return eip114x_err;

   if (eip114x_write_bytes (Instrument_ID, "STAT:QUES:VOLT?", 15) != 0)
       return eip114x_err;
   if (eip114x_read_int (Instrument_ID, Voltage_Event) != 0)
       return eip114x_err;

   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_stat_preset                                        */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_status_preset (int Instrument_ID)
{
   if (eip114x_write_bytes (Instrument_ID, "STAT:PRES", 9) != 0)
       return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_sweep1_setup                                       */
/* Warnings:                                                               */
/*                                                                         */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_sweep1_setup (int Instrument_ID, int Starting_Event, int Count,
        int Direction, int Dwell_Link, double Dwell_Time,
        double Frequency_Step, int Unit)
{
   if (eip114x_invalid_integer_range (Starting_Event, 1, 4, -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Count, 1, 32000, -3) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Direction, 0, 1, -4) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Dwell_Link, 0, 4, -5) != 0)
      return eip114x_err;
   if (eip114x_invalid_real_range (Dwell_Time, 0.100, 32000, -6) != 0)
      return eip114x_err;
   if (eip114x_invalid_real_range (Frequency_Step * unit_freq_mult[Unit],
                                    1,
                                    MaxFreq[model[Instrument_ID]], -7) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Unit, 0, 3, -8) != 0)
      return eip114x_err;

   Fmt (cmd,"%s<SWE:STAR %s;COUN %d;DIR %s;DWEL %f;DWEL:LINK %s,:SWE:STEP:FREQ %f%s",
      kw_link_trig[Starting_Event], Count, kw_dir[Direction], Dwell_Time,
      kw_link_trig[Dwell_Link], Frequency_Step, kw_unit_freq[Unit]);
   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;

   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_sweep2_setup                                       */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_sweep2_setup (int Instrument_ID, int Starting_Event, int Count,
        int Direction, int Dwell_Link, double Dwell_Time, double Power_Step)
{

   if (eip114x_invalid_integer_range (Starting_Event, 1, 4, -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Count, 1, 32000, -3) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Direction, 0, 1, -4) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Dwell_Link, 0, 4, -5) != 0)
      return eip114x_err;
   if (eip114x_invalid_real_range (Dwell_Time, 0.050, 32000, -6) != 0)
      return eip114x_err;
   if (eip114x_invalid_real_range (Power_Step,
                                   0.1,
                                   MaxPower[model[Instrument_ID]], -7) != 0)
      return eip114x_err;

   Fmt (cmd,"%s<SWE2:STAR %s;COUN %d;DIR %s;DWEL %f;DWEL:LINK %s;:SWE2:STEP:POW %f",
      kw_link_trig[Starting_Event], Count, kw_dir[Direction], Dwell_Time,
      kw_link_trig[Dwell_Link], Power_Step);
   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;

   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_sweep_freq_start_stop                              */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_sweep_freq_start_stop (int Instrument_ID, int Sweep_Limits,
        double Frequency_Start, double Frequency_Stop, int Unit)
{
   if (eip114x_invalid_integer_range (Unit, 0, 3, -4) != 0)
      return eip114x_err;
   if (Sweep_Limits == 1){
      if (eip114x_write_bytes (Instrument_ID, "FREQ:SPAN:FULL",14) != 0)
         return eip114x_err;
      }
   else {
      if (eip114x_invalid_real_range (Frequency_Start * unit_freq_mult[Unit],
                                      MinFreq[model[Instrument_ID]],
                                      MaxFreq[model[Instrument_ID]], -3) != 0)
         return eip114x_err;
      if (eip114x_invalid_real_range (Frequency_Stop * unit_freq_mult[Unit],
                                      MinFreq[model[Instrument_ID]],
                                      MaxFreq[model[Instrument_ID]], -3) != 0)
         return eip114x_err;
      Fmt (cmd,"%s<FREQ:STAR %f%s;STOP %f%s", Frequency_Start,
         kw_unit_freq[Unit], Frequency_Stop, kw_unit_freq[Unit]);
      if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
         return eip114x_err;
      }
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_sweep_power_center_span                            */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_sweep_power_center_span (int Instrument_ID, int Sweep_Limits,
        double Power_Center, double Power_Span)
{
   double MaxPowSpan;

   if (eip114x_invalid_integer_range (Sweep_Limits, 0, 1, -2) != 0)
      return eip114x_err;

   if (Sweep_Limits == 1){
      if (eip114x_write_bytes (Instrument_ID, "POW:SPAN:FULL", 13) != 0)
       return eip114x_err;
      }

   else {
      if (eip114x_invalid_real_range (Power_Center,
                                      MinPower[model[Instrument_ID]] + .1,
                                      MaxPower[model[Instrument_ID]] - .1,
                                      -3) != 0)
         return eip114x_err;

      if ((Power_Center - MinPower[model[Instrument_ID]])
          > (MaxPower[model[Instrument_ID]]) - Power_Center)
         MaxPowSpan = 2*(MaxPower[model[Instrument_ID]] - Power_Center);
      else
         MaxPowSpan = 2*(Power_Center - MinPower[model[Instrument_ID]]);

      if (eip114x_invalid_real_range (Power_Span,.2,MaxPowSpan,-4) != 0)
         return eip114x_err;
      Fmt (cmd,"%s<POW:CENT %f;SPAN %f", Power_Center, Power_Span);
      if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
         return eip114x_err;
   }

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_sweep_power_start_stop                             */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_sweep_power_start_stop (int Instrument_ID, int Sweep_Limits,
        double Power_Start, double Power_Stop)
{
   if (Sweep_Limits == 1){
      if (eip114x_write_bytes (Instrument_ID, "POW:SPAN:FULL", 13) != 0)
       return eip114x_err;
      }
   else {

      if (eip114x_invalid_real_range (Power_Start,
                                    MinPower[model[Instrument_ID]],
                                    MaxPower[model[Instrument_ID]], -3) != 0)
         return eip114x_err;
      if (eip114x_invalid_real_range (Power_Stop,
                                    MinPower[model[Instrument_ID]],
                                    MaxPower[model[Instrument_ID]], -4) != 0)
         return eip114x_err;
      Fmt (cmd,"%s<POW:STAR %f;STOP %f", Power_Start, Power_Stop);
      if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
         return eip114x_err;
   }
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_trigger_abort                                      */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_trigger_abort (int Instrument_ID, int Init_Cont_Off)
{
   if (eip114x_invalid_integer_range (Init_Cont_Off, 0, 1, -2) != 0)
      return eip114x_err;

   if (Init_Cont_Off == 1) {
      if (eip114x_write_bytes (Instrument_ID,"INIT:CONT OFF;:ABORT",20 ) != 0)
         return eip114x_err;
   }
   else {
      if (eip114x_write_bytes (Instrument_ID,"ABORT" ,5) != 0)
         return eip114x_err;
   }

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_trigger_immediate                                  */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_trigger_immediate (int Instrument_ID, int Trigger_Sequence)
{

   if (eip114x_invalid_integer_range (Trigger_Sequence, 1, 4, -2) != 0)
      return eip114x_err;
   Fmt (cmd,"%s<TRIG:SEQ%d:IMM",Trigger_Sequence);
   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;
   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_trigger_init_immediate                             */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_trigger_init_immediate (int Instrument_ID, int Initiate_Sequences)
{
   if (eip114x_invalid_integer_range (Initiate_Sequences, 0, 1, -2) != 0)
      return eip114x_err;

   if (Initiate_Sequences == 1){
      if (eip114x_write_bytes (Instrument_ID, "INIT:CONT ON", 12) != 0)
         return eip114x_err;
      }
   else{
      if (eip114x_write_bytes (Instrument_ID, "INIT:CONT OFF;:INIT",19) != 0)
         return eip114x_err;
      }

   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_trigger_setup                                      */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_trigger_setup (int Instrument_ID, int Trigger_Sequence, int Count,
        int ECount, int Source)
{

   if (eip114x_invalid_integer_range (Trigger_Sequence, 1, 4, -2) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Count,  1, 32767, -3) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (ECount, 1, 32767, -4) != 0)
      return eip114x_err;
   if (eip114x_invalid_integer_range (Source, 0, 10, -5) != 0)
      return eip114x_err;

   Fmt (cmd,"%s<TRIG:SEQ%d:COUN %d;ECO %d;SOUR %s", Trigger_Sequence, Count,
      ECount, kw_triggers[Source]);
   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;

   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}
/*=========================================================================*/
/* Name:        eip114x_trigger_signal                                     */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_trigger_signal (int Instrument_ID, int Trigger_Signal)
{
   if (eip114x_invalid_integer_range (Trigger_Signal, 1, 4, -2) != 0)
      return eip114x_err;

   Fmt (cmd,"%s<TRIG:SEQ%d:SIGNAL", Trigger_Signal);
   if (eip114x_write_bytes (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
       return eip114x_err;

   if (eip114x_test_instr_status (Instrument_ID) != 0)
      return eip114x_err;

   return eip114x_err;
}

/*=========================================================================*/
/* Name:        eip114x_user_settings                                      */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_user_settings (int Instrument_ID, int Action)
{
   int cmdname;
   if (eip114x_invalid_integer_range (Action, 0, 1, -2) != 0)
      return eip114x_err;

   if (Action == 1){
      Fmt (cmd, "%s<*SAV 0");
      if (eip114x_write_bytes (Instrument_ID, "*SAV 0", 6) != 0)
         return eip114x_err;
      }
   else{
      if (eip114x_write_bytes (Instrument_ID, "*RCL 0", 6) != 0)
         return eip114x_err;
      }

   return eip114x_err;
}

/*=========================================================================*/
/*==================== GENERAL PURPOSE ROUTINES ===========================*/
/*=========================================================================*/

/*=========================================================================*/
/* Function: Close                                                         */
/* Purpose:  This function closes the instrument.                          */
/*=========================================================================*/
int eip114x_close (int Instrument_ID)
{
    if (eip114x_invalid_integer_range (Instrument_ID, 1, eip114x_MAX_INSTR, -1) != 0)
        return eip114x_err;
    if (eip114x_device_closed (Instrument_ID))
        return eip114x_err;

    eip114x_close_instr (Instrument_ID);

    return eip114x_err;
}

/*=========================================================================*/
/* Function: Error Query                                                   */
/* Purpose:  This function queries the instrument error queue.             */
/*=========================================================================*/
int eip114x_error_query (int Instrument_ID, int *error, char *error_message)
{
   if (eip114x_invalid_integer_range (Instrument_ID, 1, eip114x_MAX_INSTR, -1) != 0)
      return eip114x_err;
   if (eip114x_device_closed(Instrument_ID) != 0)
      return eip114x_err;
   if (eip114x_write_data (Instrument_ID, "SYST:ERR?",9) != 0)
      return eip114x_err;
   Fmt (cmd,"%s<%s","");
   if (eip114x_read_data (Instrument_ID, cmd, CMDLENGTH) != 0)
      return eip114x_err;
     /* Take the error number, then take the error message
         up to, but not including a terminating line feed [t10]
         or null                                                  */
   if (Scan (cmd, "%s>%d,%s[t10][t0]", error, error_message) != 2)
      eip114x_err = 236;

   return eip114x_err;
}

/*=========================================================================*/
/* Name:        eip114x_parameter_query                                    */
/* Warnings:                                                               */
/* Last Modified November 17, 1994                                         */
/*=========================================================================*/
int eip114x_parameter_query (int Instrument_ID, char *cmd_string,
        double time, char *in_buff)
{
   long bytes_read;
   if (eip114x_invalid_integer_range (Instrument_ID, 1, eip114x_MAX_INSTR, -1) != 0)
      return eip114x_err;
   if (eip114x_device_closed(Instrument_ID) != 0)
      return eip114x_err;

   Fmt (cmd, "%s<%s", cmd_string);
   if (eip114x_write_data (Instrument_ID, cmd, NumFmtdBytes ()) != 0)
      return eip114x_err;
   Delay (time);
   bytes_read = 0L;
   if (eip114x_read_data (Instrument_ID, in_buff, 128) != 0)
      return eip114x_err;
   switch (interface[Instrument_ID]) {
      case NI_VXI: bytes_read = bytes;
                  break;
      case NI_GPIB: bytes_read = (long)ibcnt;
                  break;
         default: bytes_read = 0L;
                  break;
   }
     /* Clip off the Line Feed that the 114x always terminates      */
     /* messages with. bytes_read is NOT adjusted.                  */
   in_buff[(int)bytes_read-1] = '\0';

   return eip114x_err;
}

/*=====================================================================*/
/* Function: Revision Query                                            */
/* Purpose:  This function returns the driver and instrument revisions.*/
/*=====================================================================*/
int eip114x_revision_query (int Instrument_ID, char *model,
        char *serial_number, char *instr_rev, char *driver_rev)
{
   if (eip114x_invalid_integer_range (Instrument_ID, 1, eip114x_MAX_INSTR, -1) != 0)
      return eip114x_err;
   if (eip114x_device_closed(Instrument_ID) != 0)
      return eip114x_err;

   if (eip114x_write_data (Instrument_ID, "*IDN?", 5) != 0)
      return eip114x_err;
   if (eip114x_read_data (Instrument_ID, cmd, 100) != 0)
      return eip114x_err;

   if (Scan (cmd, "%s>%s[dxt44]%s[xt44]%s[xt44]%s[t10][t0]",
      model, serial_number,instr_rev) != 4) {
      eip114x_err = 236;
      return eip114x_err;
   }
   Fmt (driver_rev, "%s<%s", EIP114X_REVISION);

   return eip114x_err;
}

/*=========================================================================*/
/* Function: Read Instrument Buffer                                        */
/* Purpose:  This function reads the output buffer of the instrument       */
/*           This function checks that the read is valid and returns the   */
/*           actual number of characters read. The user should use this    */
/*           function rather than _read_data which it calls.               */
/* Input:    int Instrument_ID                                             */
/*           int numbytes      maximum number of bytes to be read          */
/*         char *in_buff     address of character buffer to read into      */
/* Output:   long *bytes_read  actual number of bytes read                 */
/*           returns           eip114x_err                                 */
/* Calls:    eip114x_read_data                                             */
/*=========================================================================*/
int eip114x_read (int Instrument_ID, int numbytes, char *in_buff,
        long *bytes_read)
{
   if (eip114x_invalid_integer_range (Instrument_ID, 1, eip114x_MAX_INSTR, -1) != 0)
      return eip114x_err;
   if (eip114x_device_closed(Instrument_ID) != 0)
      return eip114x_err;

   *bytes_read = 0L;
   if (eip114x_read_data (Instrument_ID, in_buff, numbytes) != 0)
      return eip114x_err;

   switch (interface[Instrument_ID]) {
      case NI_VXI: *bytes_read = bytes;
                  break;
      case NI_GPIB: *bytes_read = (long)ibcnt;
                  break;
         default: *bytes_read = 0L;
                  break;
   }
     /* Clip off the Line Feed that the 114x always terminates
      messages with. bytes_read is NOT adjusted.                  */
   in_buff[(int)*bytes_read-1] = '\0';

   return eip114x_err;
}
/*=========================================================================*/
/* Function: Reset                                                         */
/* Purpose:  This function resets the instrument.                          */
/*=========================================================================*/
int eip114x_reset (int Instrument_ID)
{
   if (eip114x_invalid_integer_range (Instrument_ID, 1, eip114x_MAX_INSTR, -1) != 0)
      return eip114x_err;
   if (eip114x_device_closed(Instrument_ID) != 0)
      return eip114x_err;
   eip114x_write_data (Instrument_ID, "*RST", 4);

   return eip114x_err;
}

/*=========================================================================*/
/* Function: Self-Test                                                     */
/* Purpose:  This function executes the instrument self-test and returns   */
/* the result. The result message may be up to 102 characters long.        */
/*=========================================================================*/
int eip114x_self_test (int Instrument_ID, int *test_result, char* test_message)
{
   int msg_num,n,tr,bit[6];

   if (eip114x_invalid_integer_range (Instrument_ID, 1, eip114x_MAX_INSTR, -1) != 0)
      return eip114x_err;
   if (eip114x_device_closed(Instrument_ID) != 0)
      return eip114x_err;
   if (eip114x_write_data (Instrument_ID, "*TST?", 5) != 0)
      return eip114x_err;
   Delay (1.0);
   if (eip114x_read_int (Instrument_ID, test_result) != 0)
      return eip114x_err;
   if (*test_result !=0) {
      tr = *test_result;
      n=32;
      msg_num=5;
      while(n > 0) {
         if ((tr-n) >= 0) {
               bit[msg_num]=TRUE;
               tr=tr-n;
         }
         else {
               bit[msg_num]=FALSE;
         }
         n=n/2;
         msg_num=msg_num-1;
      }
      Fmt(test_message,"");
      for (msg_num = 0; msg_num<6; msg_num=msg_num+1) {
         if (bit[msg_num] == TRUE)
               Fmt(test_message, "%s[a]<%s;", msg_self_test[msg_num]);
         }
   }
   else {
      Fmt(test_message, "%s<%s", "Passed");
   }

   return eip114x_err;
}
/*=========================================================================*/
/* Name:     eip114x_serial_poll                                           */
/* Description:                                                            */
/* Warnings:                                                               */
/* Last Modified: November 23, 1994                                        */
/*=========================================================================*/
int eip114x_serial_poll (int Instrument_ID, int *response)
{
   if (eip114x_device_closed(Instrument_ID) != 0)
      return eip114x_err;

   eip114x_poll (Instrument_ID, response);

   return eip114x_err;
}

/*========================================================================*/
/* Function:  eip114x_test_instr_status                                   */
/* Purpose:   Read the instrument's status byte to determine if an error  */
/*            condition exists. The function's action is enabled/disabled */
/*            by the state of the variable eip114x_auto_poll.             */
/* Returns:   eip114x_err:  -1 if Instrument_ID is invalid,               */
/*                          EIP114X_ERR_STATUS if the instrument's status */
/*                          byte has the MAV or ERRQ bit set.             */
/* Alters:    eip114x_err                                                 */
/*========================================================================*/
int eip114x_test_instr_status (int Instrument_ID)
{
   int stb;
   if (eip114x_auto_poll == TRUE) {
      if (eip114x_invalid_integer_range (Instrument_ID,1,eip114x_MAX_INSTR,-1) != 0)
         return eip114x_err;
      if (eip114x_device_closed(Instrument_ID) != 0)
         return eip114x_err;
      eip114x_poll (Instrument_ID, &stb);
      if (stb & (BIT_ERRQ | BIT_MAV) != 0) {
         eip114x_err = EIP114X_ERR_STATUS;
      }
   }

   return eip114x_err;
}

/*=========================================================================*/
/* Function: Write To Instrument                                           */
/* Purpose:  This function writes a command string of unknown length       */
/*           to the instrument after checking for valid instrument ID.     */
/*=========================================================================*/
int eip114x_write (int Instrument_ID, char *CmdString)
{
   if (eip114x_invalid_integer_range (Instrument_ID, 1, eip114x_MAX_INSTR, -1) != 0)
      return eip114x_err;
   if (eip114x_device_closed(Instrument_ID) != 0)
      return eip114x_err;

   Fmt (cmd, "%s<%s", CmdString);
   eip114x_write_data (Instrument_ID, cmd, NumFmtdBytes ());

   return eip114x_err;
}
/*=========================================================================*/
/* Function: Write Bytes                                                   */
/* Purpose:  This function writes a specified number of character data to  */
/*           the instrument. It is faster than eip114x_write, since it     */
/*           does not use the Fmt function. Unlike eip114x_write_data, it  */
/*           checks that the instrument has been opened and that the       */
/*           Instrument_ID is valid.                                       */
/*           The return value is equal to the global error variable.       */
/* Inputs:   int Instrument_ID                                             */
/*           char *buf    address of the first character to be sent        */
/*           int  cnt     the number of bytes to be written                */
/* Modified: 11/18/94                                                      */
/*=========================================================================*/
int eip114x_write_bytes (int Instrument_ID, char *buf, int cnt)
{
   eip114x_err = 0;

   if (eip114x_invalid_integer_range (Instrument_ID, 1, eip114x_MAX_INSTR, -1) != 0)
      return eip114x_err;
   if (eip114x_device_closed(Instrument_ID) != 0)
      return eip114x_err;

   switch (interface[Instrument_ID]) {
      case NI_VXI: if ((WSwrt (log_addr[Instrument_ID], buf, (long)cnt, 3, &bytes) & 0x8000) || (bytes != cnt))
                     eip114x_err = 230;
                  break;
      case NI_GPIB: if (ibwrt (bd[Instrument_ID], buf, (long)cnt) & 0x8000)
                     eip114x_err = 230;
                  break;
         default: eip114x_err = 230;
                  break;
   }

   return eip114x_err;
}

/*=========================================================================*/
/*======================== UTILITY ROUTINES ===============================*/
/*=========================================================================*/

/*=========================================================================*/
/* Function: Open Instrument                                               */
/* Purpose:  This function locates and initializes an entry in the         */
/*           Instrument Table, then initialized the VXI or GPIB driver for */
/*           this instrument. It searches the table of logical address for */
/*           a match. If one is found, it sets ID to its index, otherwise  */
/*           it adds the logical address in the table, sets ID to its      */
/*           index and adds information to the other instrument tables.    */
/*           Then, depending on the platform, it initializes either        */
/*           NI-VXI library or the NI-GPIB driver if necessary.            */
/* Inputs:   laddr is the VXI logical address of the instrument.           */
/*           platform equals 1 for NI-VXI, 2 for NI-GPIB.                  */
/*           The size of the Instrument Table can be changed in the include*/
/*           file by altering the constant eip114x_MAX_INSTR.              */
/* Outputs:  ID, the index the driver uses to identify the instrument      */
/* Returns:  the global error variable, eip114x_err.                       */
/*           Error 220 indicates that the instrument could not be found or */
/*           added to the log_addr table.                                  */
/* Globals                                                                 */
/* affected: Instrument_ID, log_addr[], base_addr[], interface[], instr_cnt*/
/*=========================================================================*/
int eip114x_open_instr (int laddr, int platform, int *ID)
{
   int i, Instrument_ID;

   Instrument_ID = 0;
   eip114x_err = 0;

  /* Check to see if the instrument is already in the Instrument Table.  */
   for (i=1; i <= eip114x_MAX_INSTR; i++)
      if (log_addr[i] == laddr) {
         Instrument_ID = i;
         i = eip114x_MAX_INSTR;
      }

  /* If it is not in the instrument table, open an entry for the instrument  */
   if (Instrument_ID <= 0)
      for (i=1; i <= eip114x_MAX_INSTR; i++)
         if (log_addr[i] == 0) {
               Instrument_ID = i;
               log_addr[Instrument_ID]  = laddr;
               base_addr[Instrument_ID] = (laddr * 0x40) + (long)(0xc000);
               interface[Instrument_ID] = platform;
               instr_cnt += 1;
               i = eip114x_MAX_INSTR;
         }

  /* If an entry could not be opened in the Instrument Table, return an error.*/
   if (Instrument_ID <= 0) {
      eip114x_err = 220;
      return eip114x_err;
   }

  /*Initialize the interface.  */
   switch (platform) {
      case NI_VXI:  if (InitVXIlibrary() < 0) {
                     eip114x_close_instr (Instrument_ID);
                     eip114x_err = 222;
                     return eip114x_err;
                     }
                     break;
      case NI_GPIB:  if (eip114x_init_gpib (Instrument_ID) != 0) {
                     eip114x_close_instr (Instrument_ID);
                     return eip114x_err;
                     }
                     break;
   }
   *ID = Instrument_ID;
   return eip114x_err;
}

/*=========================================================================*/
/* Function: Close Instrument                                              */
/* Purpose:  This function closes the instrument by removing it from the   */
/*           GPIB device table, if applicable (NI-GPIB only), and setting  */
/*           the logical address, bd, and interface to zero in the         */
/*           Instrument Table.  The return value is equal to the global    */
/*           error variable.                                               */
/*=========================================================================*/
int eip114x_close_instr (int Instrument_ID)
{
    if (interface[Instrument_ID] == NI_GPIB)
      if (bd[Instrument_ID] > 0)
         if (CloseDev (bd[Instrument_ID]) < 0) {
               eip114x_err = 221;
               return eip114x_err;
            }
    log_addr[Instrument_ID] = 0;
    bd[Instrument_ID] = 0;
    base_addr[Instrument_ID] = 0;
    interface[Instrument_ID] = 0;
    instr_cnt -= 1;

    return eip114x_err;
}

/*=========================================================================*/
/*                         NI-GPIB ONLY                                    */
/* Function: Initialize GPIB                                               */
/* Purpose:  This function initializes the GPIB interface.  First, the     */
/*           GPIB-VXI interface is initialized, then the Instrument        */
/*           interface is initialized.  The primary address of all entries */
/*           is set to 1.  The secondary address the GPIB-VXI is 0 and the */
/*           secondary address of the instrument is obtained by querying   */
/*           the GPIB-VXI.  The return value of the function is equal to   */
/*           the global error variable.                                    */
/*=========================================================================*/
int eip114x_init_gpib (int Instrument_ID)
{
   int sec_addr;
   eip114x_err = 0;

  /*  If the GPIB-VXI is not opened (bd[0]=0), open it and initialize its  */
  /*  primary and secondary addresses.                                     */

   if (bd[0] <= 0) {
      CloseInstrDevs("eip114x");
  /*  Set the interface of the GPIB-VXI.  */
      interface[0] = NI_GPIB;
      bd[0] = OpenDev ("", "eip114x");
      if (bd[0] <= 0) {
         eip114x_err = 220 ;
         return eip114x_err;
      }
      if (ibpad (bd[0], 1) & 0x8000) {
         eip114x_err = 233;
         return eip114x_err;
      }
      if (ibsad (bd[0], 0x60) & 0x8000) {
         eip114x_err = 233;
         return eip114x_err;
      }
   }

  /*  If the instrument is not opened (bd[Instrument_ID]=0), open it and  */
  /*  initialize its primary and secondary addresses.               */

   if (bd[Instrument_ID] <= 0) {
      bd[Instrument_ID] = OpenDev ("", "eip114x");
      if (bd[Instrument_ID] <= 0) {
         eip114x_err = 220 ;
         return eip114x_err;
      }
   }

  /* Query the GPIB-VXI for the secondary address of the instrument.  */
   Fmt (cmd, "%s<LaSaddr? %d", log_addr[Instrument_ID]);
   if (eip114x_write_data (0, cmd, NumFmtdBytes ()) != 0)
      return eip114x_err;

   if (eip114x_read_data (0, cmd, 50) != 0)
      return eip114x_err;
   if (Scan (cmd,"%s>%d", &sec_addr) != 1) {
      eip114x_err = 236;
      return eip114x_err;
   }
   if (ibpad (bd[Instrument_ID], 1) & 0x8000) {
      eip114x_err = 233;
      return eip114x_err;
   }
   if (sec_addr != 255) {
      if (ibsad (bd[Instrument_ID], sec_addr + 0x60) & 0x8000) {
         eip114x_err = 233;
         return eip114x_err;
      }
   }
   else
      if (ibsad (bd[Instrument_ID], 0x60) & 0x8000) {
         eip114x_err = 233;
         return eip114x_err;
      }

   if (ibconfig (bd[Instrument_ID], 6, 1) & 0x8000) {
      eip114x_err = 241;
      return eip114x_err;
   }
   return eip114x_err;
}

/*=========================================================================*/
/* Function: Invalid Integer Range                                         */
/* Purpose:  This function checks an integer to see if it lies between a   */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
int eip114x_invalid_integer_range (int val, int min, int max, int err_code)
{
   if (val < min || val > max)
      eip114x_err = err_code;
   else
      eip114x_err = 0;

   return eip114x_err;
}

/*=========================================================================*/
/* Function: Invalid Long Integer Range                                    */
/* Purpose:  This function checks a long integer to see if it lies between */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
int eip114x_invalid_longint_range (long val, long min, long max, int err_code)
{
   if (val < min || val > max)
      eip114x_err = err_code;
   else
      eip114x_err = 0;

   return eip114x_err;
}

/*=========================================================================*/
/* Function: Invalid Real Range                                            */
/* Purpose:  This function checks a real number to see if it lies between  */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
int eip114x_invalid_real_range (double val, double min, double max, int err_code)
{
   if ((val < min) || (val > max))
      eip114x_err = err_code;
   else
      eip114x_err = 0;

   return eip114x_err;
}

/*=========================================================================*/
/* Function: Device Closed                                                 */
/* Purpose:  This function checks to see if the instrument has been        */
/*           initialized.  If the device has not been opened, set the      */
/*           global error variable to 232, 0 otherwise. The return value   */
/*           is equal to the global error value.                           */
/*=========================================================================*/
int eip114x_device_closed (int Instrument_ID)
{
   if (log_addr[Instrument_ID] <= 0)
      eip114x_err = 232;
   else
      eip114x_err = 0;

   return eip114x_err;
}

/*=========================================================================*/
/* Function: Read Data                                                     */
/* Purpose:  This function reads a buffer of data from the instrument. The */
/*           return value is equal to the global error variable.           */
/*=========================================================================*/
int eip114x_read_data (int Instrument_ID, char *buf, int cnt)
{
   eip114x_err = 0;

   switch (interface[Instrument_ID]) {
      case NI_VXI:   if (WSrd (log_addr[Instrument_ID], buf, (long)cnt, 1, &bytes) & 0x8000)
                        eip114x_err = 231;
                     break;
      case NI_GPIB:  if (ibrd(bd[Instrument_ID], buf, (long)cnt) & 0x8000)
                        eip114x_err = 231;
                     break;
         default:    eip114x_err = 231;
                     break;
   }
   return eip114x_err;
}

/*=========================================================================*/
/* Function: Write Data                                                    */
/* Purpose:  This function writes a buffer of data to the instrument. The  */
/*           return value is equal to the global error variable.           */
/* Inputs:   int Instrument_ID                                             */
/*           char *buf  address of the first character to be sent          */
/*           int  cnt     the number of bytes to be written                */
/* Warnings:  This function does not first check that the write is valid.  */
/*=========================================================================*/
int eip114x_write_data (int Instrument_ID, char *buf, int cnt)
{
   eip114x_err = 0;
   switch (interface[Instrument_ID]) {
      case NI_VXI: if ((WSwrt (log_addr[Instrument_ID], buf, (long)cnt, 3, &bytes) & 0x8000) || (bytes != cnt))
                     eip114x_err = 230;
                  break;
      case NI_GPIB: if (ibwrt (bd[Instrument_ID], buf, (long)cnt) & 0x8000)
                     eip114x_err = 230;
                  break;
         default: eip114x_err = 230;
                  break;
   }
   return eip114x_err;
}

/*=========================================================================*/
/* Function: Read Data File                                                */
/* Purpose:  This function reads a buffer of data from the instrument and  */
/*           stores it to the file specified by "filename".                */
/* Inputs:   int Instrument_ID and char *filename.                         */
/*           Filename must either be a string, such as "C:\lw\instr\file"  */
/*           or a pointer to such a string.                                */
/*           The return value is equal to the global                       */
/*           error variable.                                               */
/* Warnings:  This function does not first check that the read is valid.   */
/*=========================================================================*/
int eip114x_read_data_file (int Instrument_ID, char *filename)
{
   eip114x_err = 0;

   switch (interface[Instrument_ID]) {
      case NI_VXI:   if (WSrdf (log_addr[Instrument_ID], filename, -1L, 1, &bytes) & 0x8000)
                        eip114x_err = 229;
                     break;
      case NI_GPIB:  if (ibrdf (bd[Instrument_ID], filename) & 0x8000)
                        eip114x_err = 229;
                     break;
      default:       eip114x_err = 229;
                     break;
   }
   return eip114x_err;
}

/*=========================================================================*/
/* Function: Write Data File                                               */
/* Purpose:  This function writes a buffer of data from the file specified */
/*           by "filename" to the instrument. Filename must either be a    */
/*           string, such as "C:\lw\instr\file" or a pointer to such a     */
/*           string.  The return value is equal to the global error        */
/*           variable.                                                     */
/*=========================================================================*/
int eip114x_write_data_file (int Instrument_ID, char *filename)
{
   eip114x_err = 0;

   switch (interface[Instrument_ID]) {
      case NI_VXI:   if (WSwrtf (log_addr[Instrument_ID], filename, -1L, 1, &bytes) & 0x8000)
                        eip114x_err = 228;
                     break;
      case NI_GPIB:  if (ibwrtf (bd[Instrument_ID], filename) & 0x8000)
                        eip114x_err = 228;
                     break;
         default:    eip114x_err = 228;
                     break;
   }
   return eip114x_err;
}

/*=========================================================================*/
/* Function: Read Register                                                 */
/* Purpose:  This function reads an instrument register.  The return value */
/*           is equal to the global error variable.                        */
/*=========================================================================*/
int eip114x_read_reg (int Instrument_ID, int rel_addr, int *value)
{
   eip114x_err = 0;

   switch (interface[Instrument_ID]) {
       case NI_VXI:  if (VXIinReg (log_addr[Instrument_ID], rel_addr, (unsigned short*)value) < 0)
                        eip114x_err = 231;
                     break;
      case NI_GPIB:  Fmt (cmd, "RREG? %d,%d", log_addr[Instrument_ID], rel_addr);
                     if (ibwrt(bd[0], cmd, (long)NumFmtdBytes ()) & 0x8000) {
                        eip114x_err = 230;
                        return eip114x_err;
                     }
                     if (ibrd(bd[0], cmd, 20L) & 0x8000) {
                        eip114x_err = 231;
                        return eip114x_err;
                     }
                     if (Scan (cmd, "%s>%x", value) != 1) {
                        eip114x_err = 236;
                        return eip114x_err;
                     }
                     break;
         default:    eip114x_err = 231;
                     break;
   }
   return eip114x_err;
}

/*=========================================================================*/
/* Function: Write Register                                                */
/* Purpose:  This function writes an integer value to an instrument        */
/*           register.  The return value is equal to the global error      */
/*           variable.                                                     */
/*=========================================================================*/
int eip114x_write_reg (int Instrument_ID, int rel_addr, int value)
{
   eip114x_err = 0;

   switch (interface[Instrument_ID]) {
       case NI_VXI:  if (VXIoutReg (log_addr[Instrument_ID], rel_addr, value) < 0)
                        eip114x_err = 230;
                     break;
      case NI_GPIB:  Fmt (cmd, "WREG %d,%d,%d[u]", log_addr[Instrument_ID], rel_addr, value);
                     if (ibwrt(bd[0], cmd, (long)NumFmtdBytes ()) & 0x8000)
                        eip114x_err = 230;
                     break;
           default:  eip114x_err = 230;
                     break;
    }
    return eip114x_err;
}

/*=========================================================================*/
/* Function: Set Timeout                                                   */
/* Purpose:  This function changes or disables the timeout of the device.  */
/*           Refer to array at the end of this file for timeout codes      */
/*           (index of timeout array). The return value is equal to the    */
/*           global error variable.                                        */
/*=========================================================================*/
int eip114x_set_timeout (int Instrument_ID, int tmo_code)
{
   long actual_tmo;

   eip114x_err = 0;

   switch (interface[Instrument_ID]) {
       case NI_VXI: if (WSsetTmo (timeout[tmo_code], &actual_tmo) < 0)
                       eip114x_err = 239;
                    break;
      case NI_GPIB: if (ibtmo (bd[Instrument_ID], tmo_code) & 0x8000)
                       eip114x_err = 239;
                    else {
                       actual_tmo = iberr;
                       eip114x_err = 0;
                    }
                    break;
           default: eip114x_err = 239;
                    break;
   }
   return eip114x_err;
}

/*=========================================================================*/
/* Function: Poll                                                          */
/* Purpose:  This function performs a serial poll on the instrument.  The  */
/*           status byte of the instrument is placed in the response       */
/*           variable. The return value is the global error variable.      */
/*=========================================================================*/
int eip114x_poll (int Instrument_ID, int *response)
{
   eip114x_err = 0;
   switch (interface[Instrument_ID]) {
       case NI_VXI: if (WScmd (log_addr[Instrument_ID], 0xcfff, 1, (unsigned short*)response) & 0x8000)
                       eip114x_err = 226;
                    break;
      case NI_GPIB: if (ibrsp (bd[Instrument_ID], (char*)response) & 0x8000)
                       eip114x_err = 226;
                    break;
           default: eip114x_err = 226;
                    break;
   }
   *response &= 0xff;
   return eip114x_err;
}

/*=========================================================================*/
/* Function: Get Status Register                                           */
/* Purpose:  This function reads the VXI status register of the instrument */
/*           which is located at the instrument's base_addr + 4.           */
/*           Note that this is not the same value read by the poll function*/
/*           The return value is the global error variable.                */
/*=========================================================================*/
int eip114x_get_status_reg (int Instrument_ID, int *response)
{
   if (eip114x_read_reg (Instrument_ID, 4, response) != 0)
       eip114x_err = 226;
   else
       eip114x_err = 0;

   return eip114x_err;
}

/*=========================================================================*/
/* Function: Setup Arrays                                                  */
/* Purpose:  This function is called by the init routine to initialize     */
/*           static arrays.                                                */
/*           When formatting instrument command strings, these arrays can  */
/*           be used to simplify your code. e.g.                           */
/*                                                                         */
/*                Fmt (cmd, "%s<MEAS:%s", function[func]);                 */
/*                                                                         */
/*           The variable func is being used as an indexing parameter that */
/*           can be passed from a function panel.                          */
/*                                                                         */
/*=========================================================================*/
void eip114x_setup_arrays ()
{
/*  Timeout array values are equivalent to timeout in milliseconds.  */
   timeout[0]  = 0xffffffff; /*  Equivalent to turning timeout off  */
   timeout[1]  = 1;
   timeout[2]  = 1;
   timeout[3]  = 1;
   timeout[4]  = 1;
   timeout[5]  = 1;
   timeout[6]  = 3;
   timeout[7]  = 10;
   timeout[8]  = 30;
   timeout[9]  = 100;
   timeout[10] = 300;
   timeout[11] = 1000;
   timeout[12] = 3000;
   timeout[13] = 10000;
   timeout[14] = 30000;
   timeout[15] = 100000;
   timeout[16] = 300000;
   timeout[17] = 1000000;

   MaxFreq [EIP_MODEL_1140]  =  20.0E9;
   MinFreq [EIP_MODEL_1140]  =  10.0E6;
   MaxPower[EIP_MODEL_1140]  =  15.0;
   MinPower[EIP_MODEL_1140]  = -90.0;
   MaxAtten[EIP_MODEL_1140]  =  90.0;
   MinAtten[EIP_MODEL_1140]  =   0.0;

   MaxFreq [EIP_MODEL_1141]  = 20.0E9;
   MinFreq [EIP_MODEL_1141]  =  2.0E9;
   MaxPower[EIP_MODEL_1141]  =  15.0;
   MinPower[EIP_MODEL_1141]  = -90.0;
   MaxAtten[EIP_MODEL_1141]  =  90.0;
   MinAtten[EIP_MODEL_1141]  =   0.0;

   MaxFreq [EIP_MODEL_1142]  = 12.4E9;
   MinFreq [EIP_MODEL_1142]  =  4.0E9;
   MaxPower[EIP_MODEL_1142]  =  15.0;
   MinPower[EIP_MODEL_1142]  =-100.0;
   MaxAtten[EIP_MODEL_1142]  = 100.0;
   MinAtten[EIP_MODEL_1142]  =   0.0;

   msg_self_test[0] = "RAM fault";
   msg_self_test[1] = "Invalid power cal cksum";
   msg_self_test[2] = "Voltage fault";
   msg_self_test[3] = "Invalid YIG cal data";
   msg_self_test[4] = "Invalid power cal data";
   msg_self_test[5] = "YTO fault";

   msg_cal[0] = "YTO Cal Freqs invalid";
   msg_cal[1] = "YIG Cal failed";
   msg_cal[2] = "YIG Cal data storage failed";
   msg_cal[3] = "YIG init incomplete";
   msg_cal[4] = "Voltage fault";

   kw_mode[1]="CW";
   kw_mode[2]="LIST";
   kw_mode[3]="SWE";
   kw_mode[4]="SWE2";

   kw_binary[0]="0";
   kw_binary[1]="1";

   kw_dir[0]="DOWN";
   kw_dir[1]="UP";

   kw_ext_int[0]="INT";
   kw_ext_int[1]="EXT";

   kw_link_trig[0]="DWEL";
   kw_link_trig[1]="TRIG:SEQ";
   kw_link_trig[2]="TRIG:SEQ2";
   kw_link_trig[3]="TRIG:SEQ3";
   kw_link_trig[4]="TRIG:SEQ4";

     /* Alternate Numeric Value Keywords */
   kw_nv_alt[0]="";
   kw_nv_alt[1]="DOWN";
   kw_nv_alt[2]="UP";
   kw_nv_alt[3]="MIN";
   kw_nv_alt[4]="MAX";

   kw_triggers[0]  ="BUS";
   kw_triggers[1]  ="ECLT0";
   kw_triggers[2]  ="ECLT1";
   kw_triggers[3]  ="TTLT0";
   kw_triggers[4]  ="TTLT1";
   kw_triggers[5]  ="TTLT2";
   kw_triggers[6]  ="TTLT3";
   kw_triggers[7]  ="TTLT4";
   kw_triggers[8]  ="TTLT5";
   kw_triggers[9]  ="TTLT6";
   kw_triggers[10] ="TTLT7";

   kw_unit_freq[0]="Hz";
   kw_unit_freq[1]="KHz";
   kw_unit_freq[2]="MHz";
   kw_unit_freq[3]="GHz";

   unit_freq_mult[0]=1.0;
   unit_freq_mult[1]=1.0E3;
   unit_freq_mult[2]=1.0E6;
   unit_freq_mult[3]=1.0E9;
}

/*=========================================================================*/
/* Function: eip114x_read_int                                              */
/* Purpose:  Read the instrument's output buffer and convert the reply to  */
/*           and integer.                                                  */
/*           Uses global array:  cmd                                       */
/*=========================================================================*/
int eip114x_read_int (int Instrument_ID, int *result)
{
   if (eip114x_read_data (Instrument_ID, cmd, 20) == 0)
      if (Scan (cmd, "%s>%d", result) != 1)
         eip114x_err = 236;

   return eip114x_err;
}

/*=== END OF eip114x.c ====================================================*/
